<?php

declare(strict_types = 1);

namespace Drupal\email_hidden_formatter\Controller;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\node\NodeInterface;

/**
 * Controller for the Email Hidden Formatter module.
 */
class EmailHiddenController extends ControllerBase {

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected RendererInterface $renderer;

  /**
   * The entity display repository service.
   *
   * @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface
   */
  protected EntityDisplayRepositoryInterface $entityDisplayRepository;

  /**
   * Constructs a new EmailHiddenController object.
   *
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   * @param \Drupal\Core\Entity\EntityDisplayRepositoryInterface $entity_display_repository
   *   The entity display repository service.
   */
  public function __construct(RendererInterface $renderer, EntityDisplayRepositoryInterface $entity_display_repository) {
    $this->renderer = $renderer;
    $this->entityDisplayRepository = $entity_display_repository;
  }

  /**
   * Builds an AJAX response to dynamically render the email field.
   *
   * @param \Drupal\node\NodeInterface $node
   *   The node entity containing the email field.
   * @param string $field_name
   *   The machine name of the field to render.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   The AJAX response object containing the rendered email field.
   */
  public function process(NodeInterface $node, string $field_name, string $label): AjaxResponse {

    $field = $node->get($field_name);

    if (!$field || $field->isEmpty()) {
      $rendered_field = $this->t('Field is empty.');
    }
    else {
      $render_array = $field->view([
        'label' => $label,
      ]);
      $rendered_field = $this->renderer->render($render_array);
    }

    $css_safe_field_name = str_replace('_', '-', $field_name);

    $response = new AjaxResponse();
    $response->addCommand(new ReplaceCommand('.field--name-' . $css_safe_field_name, $rendered_field));

    return $response;
  }

}
