<?php

declare(strict_types = 1);

namespace Drupal\email_hidden_formatter\Plugin\Field\FieldFormatter;

use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'Email Hidden' formatter.
 *
 * Provides a field formatter that hides the email field by default
 * and allows it to be revealed dynamically via an AJAX request.
 *
 * @FieldFormatter(
 *   id = "email_hidden",
 *   label = @Translation("Email Hidden"),
 *   field_types = {"email"},
 * )
 */
class EmailHiddenFormatter extends FormatterBase implements ContainerFactoryPluginInterface {

  /**
   * The entity display repository
   *
   * @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface
   */
  protected $entityDisplayRepository;

  /**
   * Constructs a StringFormatter instance.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Any third party settings settings.
   * @param \Drupal\Core\Entity\EntityDisplayRepositoryInterface $entity_display_repository
   *   The entity display repository.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, EntityDisplayRepositoryInterface $entity_display_repository) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    $this->entityDisplayRepository = $entity_display_repository;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('entity_display.repository')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings(): array {
    // Define default settings for the formatter.
    return [
        'title' => 'Show email',
      ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    // Create a settings form for the formatter.
    $elements['title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Link title'),
      '#description' => $this->t('Enter the text to display as the link.'),
      '#default_value' => $this->getSetting('title'),
    ];
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary(): array {
    // Provide a summary of the settings.
    $summary = [];
    $summary[] = $this->t('Link title: @title', ['@title' => $this->getSetting('title')]);
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode): array {
    $elements = [];
    $field_name = $items->getName();
    $entity = $items->getEntity();

    $view_mode = $this->viewMode;
    $view_display = $this->entityDisplayRepository->getViewDisplay($entity->getEntityTypeId(), $entity->bundle(), $view_mode);

    $label_visibility = 'above';
    if ($view_display && $view_display->getComponent($field_name)) {

      $field_component = $view_display->getComponent($field_name);

      $label_visibility = $field_component['label'] ?? 'above'; // Możliwe wartości: hidden, inline, above.
    }

    foreach ($items as $delta => $item) {
      $elements[$delta] = [
        '#type' => 'link',
        '#title' => $this->getSetting('title'),
        '#url' => Url::fromRoute('email_hidden_formatter.email_hidden', [
          'node' => $entity->id(),
          'field_name' => $field_name,
          'label' => $label_visibility,
        ]),
        '#attributes' => [
          'class' => ['use-ajax'],
          'data-custom' => 'email-hidden-formatter',
        ],
      ];
    }

    return $elements;
  }
}
